@@ -4,7 +4,6 @@
 # https://www.sphinx-doc.org/en/master/usage/configuration.html
 from __future__ import annotations
 
-import os
 from importlib.metadata import version as _version
 
 # -- Project information -----------------------------------------------------
@@ -24,7 +23,11 @@ extensions = [
     "sphinx.ext.coverage",
     "sphinx.ext.viewcode",
     "sphinx.ext.todo",
-    "sphinx_github_changelog",
+    "myst_parser",
+]
+
+myst_enable_extensions = [
+    "colon_fence",
 ]
 
 templates_path = ["_templates"]
@@ -36,4 +39,3 @@ exclude_patterns: list[str] = []
 html_theme = "sphinx_rtd_theme"
 html_static_path = ["_static"]
 master_doc = "index"
-sphinx_github_changelog_token = os.environ.get("CHANGELOG_GITHUB_TOKEN")
