@@ -4,7 +4,7 @@
 from __future__ import annotations
 
 import logging
-from typing import Any
+from typing import Any, ClassVar
 
 from ring_doorbell.const import (
     CHIME_KINDS,
@@ -34,13 +34,12 @@ class RingChime(RingGeneric):
         """Return Ring device family type."""
         return "chimes"
 
-    def update_health_data(self) -> None:
+    async def async_update_health_data(self) -> None:
         """Update health attrs."""
-        self._health_attrs = (
-            self._ring.query(HEALTH_CHIMES_ENDPOINT.format(self.device_api_id))
-            .json()
-            .get("device_health", {})
+        resp = await self._ring.async_query(
+            HEALTH_CHIMES_ENDPOINT.format(self.device_api_id)
         )
+        self._health_attrs = resp.json().get("device_health", {})
 
     @property
     def model(self) -> str:
@@ -58,17 +57,15 @@ class RingChime(RingGeneric):
             if isinstance(capability, RingCapability)
             else RingCapability.from_name(capability)
         )
-        if capability == RingCapability.VOLUME:
-            return True
-        return False
+        return capability == RingCapability.VOLUME
 
     @property
     def volume(self) -> int:
-        """Return if chime volume."""
+        """Return the chime volume."""
         return self._attrs["settings"].get("volume", 0)
 
-    @volume.setter
-    def volume(self, value: int) -> None:
+    async def async_set_volume(self, value: int) -> None:
+        """Set the chime volume."""
         if not ((isinstance(value, int)) and (CHIME_VOL_MIN <= value <= CHIME_VOL_MAX)):
             raise RingError(MSG_VOL_OUTBOUND.format(CHIME_VOL_MIN, CHIME_VOL_MAX))
 
@@ -77,20 +74,37 @@ class RingChime(RingGeneric):
             "chime[settings][volume]": str(value),
         }
         url = CHIMES_ENDPOINT.format(self.device_api_id)
-        self._ring.query(url, extra_params=params, method="PUT")
-        self._ring.update_devices()
+        await self._ring.async_query(url, extra_params=params, method="PUT")
 
-    @property
-    def linked_tree(self) -> dict[str, Any]:
+    async def async_get_linked_tree(self) -> dict[str, Any]:
         """Return doorbell data linked to chime."""
         url = LINKED_CHIMES_ENDPOINT.format(self.device_api_id)
-        return self._ring.query(url).json()
+        resp = await self._ring.async_query(url)
+        return resp.json()
 
-    def test_sound(self, kind: RingEventKind | str = RingEventKind.DING) -> bool:
+    async def async_test_sound(
+        self, kind: RingEventKind | str = RingEventKind.DING
+    ) -> bool:
         """Play chime to test sound."""
         kind_str = kind.value if isinstance(kind, RingEventKind) else kind
         if kind_str not in CHIME_TEST_SOUND_KINDS:
             return False
         url = TESTSOUND_CHIME_ENDPOINT.format(self.device_api_id)
-        self._ring.query(url, method="POST", extra_params={"kind": kind_str})
+        await self._ring.async_query(
+            url, method="POST", extra_params={"kind": kind_str}
+        )
         return True
+
+    DEPRECATED_API_QUERIES: ClassVar = {
+        *RingGeneric.DEPRECATED_API_QUERIES,
+        "update_health_data",
+        "test_sound",
+    }
+    DEPRECATED_API_PROPERTY_GETTERS: ClassVar = {
+        *RingGeneric.DEPRECATED_API_PROPERTY_GETTERS,
+        "linked_tree",
+    }
+    DEPRECATED_API_PROPERTY_SETTERS: ClassVar = {
+        *RingGeneric.DEPRECATED_API_PROPERTY_SETTERS,
+        "volume",
+    }
