@@ -11,10 +11,10 @@ import json
 import logging
 import select
 import sys
-import traceback
+from contextlib import asynccontextmanager
 from datetime import datetime
 from pathlib import Path, PurePath
-from typing import Sequence, cast
+from typing import Sequence, cast, TypeVar, NoReturn
 
 import asyncclick as click
 
@@ -26,9 +26,17 @@ from ring_doorbell import (
     RingDoorBell,
     RingEvent,
     RingGeneric,
+    RingStickUpCam,
+    RingOther,
+    RingCapability,
+)
+from ring_doorbell.const import (
+    CLI_TOKEN_FILE,
+    GCM_TOKEN_FILE,
+    PACKAGE_NAME,
+    USER_AGENT,
+    DOORBELL_EXISTING_TYPE,
 )
-from ring_doorbell.const import CLI_TOKEN_FILE, GCM_TOKEN_FILE, PACKAGE_NAME, USER_AGENT
-from ring_doorbell.listen import can_listen
 
 
 def _header() -> None:
@@ -40,31 +48,65 @@ def _bar() -> None:
     echo("---------------------------------")
 
 
+def error(msg: str) -> NoReturn:
+    """Print an error and exit."""
+    echo(msg)
+    sys.exit(1)
+
+
 click.anyio_backend = "asyncio"  # type: ignore[attr-defined]
 
 pass_ring = click.make_pass_decorator(Ring)
+pass_doorbell = click.make_pass_decorator(RingDoorBell)
 
 cache_file = Path(CLI_TOKEN_FILE)
 gcm_cache_file = Path(GCM_TOKEN_FILE)
 
 
-class ExceptionHandlerGroup(click.Group):
-    """Group to capture all exceptions and echo them nicely.
+def CatchAllExceptions(cls):
+    """Capture all exceptions and prints them nicely.
 
-    Idea from https://stackoverflow.com/a/44347763
+    Idea from https://stackoverflow.com/a/44347763 and
+    https://stackoverflow.com/questions/52213375
     """
 
-    def __call__(self, *args, **kwargs):
-        """Run the coroutine in the event loop and echo any exceptions."""
-        try:
-            asyncio.run(self.main(*args, **kwargs))
-        except asyncio.CancelledError:
-            pass
-        except KeyboardInterrupt:
-            echo("Cli interrupted with keyboard interrupt")
-        except Exception as ex:  # pylint: disable=broad-exception-caught
-            echo(f"Got error: {ex!r}")
-            traceback.print_exc()
+    def _handle_exception(debug, exc):
+        if isinstance(exc, click.ClickException):
+            raise
+        # Handle exit request from click.
+        if isinstance(exc, click.exceptions.Exit):
+            sys.exit(exc.exit_code)
+        echo(f"Raised error: {exc}")
+        if debug:
+            raise
+        echo("Run with --debug enabled to see stacktrace")
+        sys.exit(1)
+
+    class _CommandCls(cls):
+        _debug = False
+
+        async def make_context(self, info_name, args, parent=None, **extra):
+            self._debug = any(
+                [arg for arg in args if arg in ["--debug", "-d", "--verbose", "-v"]]
+            )
+            try:
+                return await super().make_context(
+                    info_name, args, parent=parent, **extra
+                )
+            except Exception as exc:
+                _handle_exception(self._debug, exc)
+
+        async def invoke(self, ctx):
+            try:
+                return await super().invoke(ctx)
+            except asyncio.CancelledError:
+                pass
+            except KeyboardInterrupt:
+                echo("Cli interrupted with keyboard interrupt")
+            except Exception as exc:
+                _handle_exception(self._debug, exc)
+
+    return _CommandCls
 
 
 class MutuallyExclusiveOption(click.Option):
@@ -113,7 +155,7 @@ def _format_filename(device_name, event):
     return filename.replace(" ", "_").replace(":", ".") + ".mp4"
 
 
-def _do_auth(username, password, user_agent=USER_AGENT):
+async def _do_auth(username, password, user_agent=USER_AGENT):
     if not username:
         username = input("Username: ")
 
@@ -122,14 +164,14 @@ def _do_auth(username, password, user_agent=USER_AGENT):
 
     auth = Auth(user_agent, None, token_updated)
     try:
-        auth.fetch_token(username, password)
+        await auth.async_fetch_token(username, password)
         return auth
     except Requires2FAError:
-        auth.fetch_token(username, password, input("2FA Code: "))
+        await auth.async_fetch_token(username, password, input("2FA Code: "))
         return auth
 
 
-def _get_ring(username, password, do_update_data, user_agent=USER_AGENT):
+async def _get_ring(username, password, do_update_data, user_agent=USER_AGENT):
     # connect to Ring account
     global cache_file, gcm_cache_file
     if user_agent != USER_AGENT:
@@ -142,26 +184,71 @@ def _get_ring(username, password, do_update_data, user_agent=USER_AGENT):
             token_updated,
         )
         ring = Ring(auth)
-        do_method = ring.update_data if do_update_data else ring.create_session
+        do_method = (
+            ring.async_update_data if do_update_data else ring.async_create_session
+        )
         try:
-            do_method()
+            await do_method()
         except AuthenticationError:
-            auth = _do_auth(username, password)
+            auth = await _do_auth(username, password)
             ring = Ring(auth)
-            do_method = ring.update_data if do_update_data else ring.create_session
-            do_method()
+            do_method = (
+                ring.async_update_data if do_update_data else ring.async_create_session
+            )
+            await do_method()
     else:
-        auth = _do_auth(username, password, user_agent=user_agent)
+        auth = await _do_auth(username, password, user_agent=user_agent)
         ring = Ring(auth)
-        do_method = ring.update_data if do_update_data else ring.create_session
-        do_method()
+        do_method = (
+            ring.async_update_data if do_update_data else ring.async_create_session
+        )
+        await do_method()
 
     return ring
 
 
+_T = TypeVar("_T")
+
+
+def _get_device(
+    ring: Ring,
+    device_families: list[str],
+    device_type: type[_T],
+    device_name: str | None = None,
+    *,
+    device_description: str | None = None,
+) -> _T:
+    description = (
+        device_description if device_description else " or ".join(device_families)
+    )
+    if not device_name:
+        dev_dict: dict[int, RingGeneric] = {}
+        devices = ring.devices()
+        for device_family in device_families:
+            for dev in devices[device_family]:
+                dev_dict[dev.device_api_id] = dev
+        devs = list(dev_dict.values())
+        found = len(dev_dict)
+        if found == 1:
+            return cast(_T, devs[0])
+        elif found == 0:
+            error(f"No {description} found")
+        else:
+            error(
+                f"There are {found} {description}s, you need to pass the --device-name option."
+            )
+    elif device := ring.get_device_by_name(device_name):
+        if device.family in device_families and isinstance(device, device_type):
+            return device
+        else:
+            error(f"{device_name} is not a {description}")
+    else:
+        error(f"Cannot find {description} with name {device_name}")
+
+
 @click.group(
     invoke_without_command=True,
-    cls=ExceptionHandlerGroup,
+    cls=CatchAllExceptions(click.Group),
 )
 @click.version_option(package_name="ring_doorbell")
 @click.option(
@@ -189,21 +276,34 @@ def _get_ring(username, password, do_update_data, user_agent=USER_AGENT):
 @click.pass_context
 async def cli(ctx, username, password, debug, user_agent):
     """Command line function."""
+    # no need to perform any checks if we are just displaying the help
+    if "--help" in sys.argv:
+        # Context object is required to avoid crashing on sub-groups
+        ctx.obj = Ring(None)
+        return
     _header()
 
     logging.basicConfig()
     log_level = logging.DEBUG if debug else logging.INFO
     logger = logging.getLogger(PACKAGE_NAME)
     logger.setLevel(log_level)
-    if can_listen:
-        logger = logging.getLogger("firebase_messaging")
-        logger.setLevel(log_level)
+
+    logger = logging.getLogger("firebase_messaging")
+    logger.setLevel(log_level)
 
     no_update_commands = ["listen"]
     no_update = ctx.invoked_subcommand in no_update_commands
 
-    ring = _get_ring(username, password, not no_update, user_agent)
-    ctx.obj = ring
+    @asynccontextmanager
+    async def async_wrapped_ring(ring: Ring):
+        try:
+            yield ring
+        finally:
+            await ring.auth.async_close()
+
+    ring = await _get_ring(username, password, not no_update, user_agent)
+    # wrapped ring will ensure async_close is called when cli is finished
+    ctx.obj = await ctx.with_async_resource(async_wrapped_ring(ring))
 
     if ctx.invoked_subcommand is None:
         return await ctx.invoke(show)
@@ -257,7 +357,7 @@ async def list_command(ring: Ring) -> None:
     help="Name of the ring device",
 )
 async def motion_detection(ctx, ring: Ring, device_name, turn_on, turn_off):
-    """Display ring devices."""
+    """Get and change the motion detecton status of a device."""
     device = ring.get_device_by_name(device_name)
 
     if not device:
@@ -279,12 +379,54 @@ async def motion_detection(ctx, ring: Ring, device_name, turn_on, turn_off):
         echo(f"{device!s} already has motion detection {state}")
         return None
 
-    device.motion_detection = turn_on if turn_on else False
+    await device.async_set_motion_detection(turn_on if turn_on else False)
+    await ring.async_update_devices()
     state = "on" if device.motion_detection else "off"
     echo(f"{device!s} motion detection set to {state}")
     return None
 
 
+@cli.command()
+@pass_ring
+@click.pass_context
+@click.argument("enable", type=click.BOOL, default=None, required=False)
+@click.option(
+    "--device-name",
+    "-dn",
+    required=True,
+    default=None,
+    help="Name of the ring device",
+)
+async def light(ctx, ring: Ring, device_name, enable):
+    """Get and change the light state of a device."""
+    device = ring.get_device_by_name(device_name)
+
+    if not device:
+        echo(
+            f"No device with name {device_name} found."
+            + " List of found device names (kind) is:"
+        )
+        return await ctx.invoke(list_command)
+    if not device.has_capability(RingCapability.LIGHT):
+        echo(f"{device!s} does not have a light")
+        return None
+    device = cast(RingStickUpCam, device)
+    state = "on" if device.light else "off"
+    if enable is None:
+        echo(f"{device!s} light is {state}")
+        return None
+    is_on = device.light
+    if (enable and is_on) or (not enable and not is_on):
+        echo(f"{device!s} light is already {state}")
+        return None
+
+    await device.async_set_light(enable)
+    await ring.async_update_devices()
+    state = "on" if device.light else "off"
+    echo(f"{device!s} light set to {state}")
+    return None
+
+
 @cli.command()
 @click.option(
     "--device-name",
@@ -311,7 +453,7 @@ async def show(ctx, ring: Ring, device_name):
         devices = ring.get_device_list()
 
     for dev in devices:
-        dev.update_health_data()
+        await dev.async_update_health_data()
         echo("Name:       %s" % dev.name)
         echo("Family:     %s" % dev.family)
         echo("ID:         %s" % dev.id)
@@ -407,7 +549,7 @@ async def groups(ring: Ring, json_flag) -> None:
         echo("No ring device groups setup")
     else:
         for light_group in ring.groups().values():
-            light_group.update()
+            await light_group.async_update()
             echo(json.dumps(light_group._attrs, indent=2))
             echo(json.dumps(light_group._health_attrs, indent=2))
 
@@ -422,7 +564,8 @@ async def groups(ring: Ring, json_flag) -> None:
 @pass_ring
 async def raw_query(ring: Ring, url) -> None:
     """Directly query a url and return json result."""
-    data = ring.query(url).json()
+    resp = await ring.async_query(url)
+    data = resp.json()
     echo(json.dumps(data, indent=2))
 
 
@@ -471,7 +614,7 @@ async def history_command(ctx, ring: Ring, device_name, kind, limit, json_flag):
         )
         return await ctx.invoke(list_command)
 
-    history = device.history(limit=limit, kind=kind, convert_timezone=False)
+    history = await device.async_history(limit=limit, kind=kind, convert_timezone=False)
     echo(json.dumps(history, indent=2))
     return None
 
@@ -515,7 +658,7 @@ async def history_command(ctx, ring: Ring, device_name, kind, limit, json_flag):
     "-dn",
     default=None,
     required=False,
-    help="Name of the ring device, if ommited uses the first device returned",
+    help="Name of the ring device, if omitted uses the first device returned",
 )
 @pass_ring
 @click.pass_context
@@ -550,8 +693,8 @@ async def videos(
         not count
         and not download
         and not download_all
-        and device.last_recording_id
-        and (url := device.recording_url(device.last_recording_id))
+        and (last_recording_id := await device.async_get_last_recording_id())
+        and (url := await device.async_recording_url(last_recording_id))
     ):
         echo("Last recording url is: " + url)
         return None
@@ -561,15 +704,17 @@ async def videos(
         download = True
         max_count = -1
 
-    def _get_events(device, max_count):
+    async def _get_events(device, max_count):
         limit = 100 if max_count == -1 else min(100, max_count)
         events = []
-        history = device.history(limit=limit)
+        history = await device.async_history(limit=limit)
         while len(history) > 0:
             events += history
             if (len(events) >= max_count and max_count != -1) or len(history) < limit:
                 break
-            history = device.history(older_than=history[-1]["id"], limit=limit)
+            history = await device.async_history(
+                older_than=history[-1]["id"], limit=limit
+            )
         return events
 
     if count:
@@ -578,7 +723,7 @@ async def videos(
             + "\tThis may take some time....\n"
         )
 
-        events = _get_events(device, max_count)
+        events = await _get_events(device, max_count)
 
         motion = len([m["kind"] for m in events if m["kind"] == "motion"])
         ding = len([m["kind"] for m in events if m["kind"] == "ding"])
@@ -595,7 +740,7 @@ async def videos(
                 "\tGetting videos linked on your Ring account.\n"
                 "\tThis may take some time....\n"
             )
-            events = _get_events(device, max_count)
+            events = await _get_events(device, max_count)
 
         echo(
             f"\tDownloading {len(events)} videos linked on your Ring account.\n"
@@ -605,32 +750,120 @@ async def videos(
             filename = str(PurePath(download_to, _format_filename(device.name, event)))
             echo(f"\t{counter}/{len(events)} Downloading {filename}")
 
-            device.recording_download(event["id"], filename=filename, override=False)
+            await device.async_recording_download(
+                event["id"], filename=filename, override=False
+            )
         return None
     return None
 
 
+@cli.group(invoke_without_command=True)
+@pass_ring
+@click.pass_context
+@click.option(
+    "--device-name",
+    "-dn",
+    required=False,
+    default=None,
+    help="Name of the ring device",
+)
+async def in_home_chime(ctx, ring: Ring, device_name):
+    """View and manage the Doorbell in-home chime. To see the current in-home chime status of a device, only pass the device name."""
+    if "--help" in sys.argv:
+        return
+    device = _get_device(
+        ring,
+        ["authorized_doorbots", "doorbots"],
+        RingDoorBell,
+        device_name,
+        device_description="doorbell",
+    )
+
+    if ctx.invoked_subcommand is None:
+        echo("Name:       %s" % device.name)
+        echo("ID:         %s" % device.id)
+        echo("Type:       %s" % device.existing_doorbell_type)
+        echo("Enabled:    %s" % device.existing_doorbell_type_enabled)
+        echo("Duration:   %s" % device.existing_doorbell_type_duration)
+        return None
+
+    ctx.obj = device
+
+
+@in_home_chime.command(name="type")
+@pass_doorbell
+@click.pass_context
+@click.argument(
+    "new_type",
+    type=click.Choice(list(DOORBELL_EXISTING_TYPE.values()), case_sensitive=False),
+    default=None,
+    required=False,
+)
+async def in_home_chime_type(ctx, device: RingDoorBell, new_type):
+    """Get/set the type of In-home chime."""
+    if new_type is None:
+        echo(device.existing_doorbell_type)
+        return
+
+    if device.family == "authorized_doorbots":
+        exit(
+            f"{device.name} is a shared device and you do not have permission to update this value"
+        )
+    new_type_int = next(k for k, v in DOORBELL_EXISTING_TYPE.items() if v == new_type)
+
+    await device.async_set_existing_doorbell_type(new_type_int)
+    echo(f"{device.name}'s in-home chime type has been set to {new_type}")
+
+
+@in_home_chime.command()
+@pass_doorbell
+@click.pass_context
+@click.argument("enable", type=click.BOOL, default=None, required=False)
+async def enabled(ctx, device: RingDoorBell, enable: bool | None):
+    """Gets/sets the in-home chime enabled status.
+
+    ENABLE: 1/0, true/false, t/f, yes/no, y/n, and on/off
+    """
+    if enable is None:
+        echo(device.existing_doorbell_type_enabled)
+        return
+
+    if device.family == "authorized_doorbots":
+        exit(
+            f"{device.name} is a shared device and you do not have permission to update this value"
+        )
+    await device.async_set_existing_doorbell_type_enabled(enable)
+    echo(
+        f"{device.name}'s in-home chime has been {'enabled' if enable else 'disabled'}"
+    )
+
+
+@in_home_chime.command()
+@pass_doorbell
+@click.pass_context
+@click.argument("duration", type=click.IntRange(0, 100), default=None, required=False)
+async def duration(ctx, device: RingDoorBell, duration: int | None):
+    """Gets/sets the in-home chime duration.
+
+    DURATION: Value between 0 and 100
+    """
+    if duration is None:
+        echo(device.existing_doorbell_type_duration)
+        return
+
+    if device.family == "authorized_doorbots":
+        exit(
+            f"{device.name} is a shared device and you do not have permission to update this value"
+        )
+    await device.async_set_existing_doorbell_type_duration(int(duration))
+    echo(f"{device.name}'s in-home chime duration has been set to {duration} seconds")
+
+
 async def ainput(string: str):
     loop = asyncio.get_event_loop()
     await loop.run_in_executor(None, lambda s=string: sys.stdout.write(s + " "))  # type: ignore[misc]
 
-    def read_with_timeout(timeout):
-        if select.select(
-            [
-                sys.stdin,
-            ],
-            [],
-            [],
-            timeout,
-        )[0]:
-            # line = sys.stdin.next()
-            return sys.stdin.readline()
-        return None
-
-    line = None
-    while loop.is_running() and not line:
-        line = await loop.run_in_executor(None, functools.partial(read_with_timeout, 1))
-    return line
+    return await loop.run_in_executor(None, sys.stdin.readline)
 
 
 def get_now_str():
@@ -683,10 +916,6 @@ async def listen(
     show_credentials,
 ) -> None:
     """Listen to push notification like the ones sent to your phone."""
-    if not can_listen:
-        echo("Ring is not configured for listening to notifications!")
-        echo("pip install ring_doorbell[listen]")
-        return
 
     from ring_doorbell.listen import (  # pylint:disable=import-outside-toplevel
         RingEventListener,
@@ -713,12 +942,34 @@ async def listen(
             credentials = json.load(f)
 
     event_listener = RingEventListener(ring, credentials, credentials_updated_callback)
-    event_listener.start()
+    await event_listener.start()
     event_listener.add_notification_callback(_event_handler(ring).on_event)
 
     await ainput("Listening, press enter to cancel\n")
 
-    event_listener.stop()
+    await event_listener.stop()
+
+
+@cli.command
+@click.option(
+    "--device-name",
+    required=False,
+    default=None,
+    help=("Name of the intercom if there are more than one."),
+)
+@pass_ring
+@click.pass_context
+async def open_door(ctx, ring: Ring, device_name: str | None) -> None:
+    """Open the door of a intercom device."""
+    device = _get_device(
+        ring,
+        ["intercoms", "other"],
+        RingOther,
+        device_name,
+        device_description="intercom",
+    )
+    await device.async_open_door()
+    echo(f"{device.name} opened")
 
 
 if __name__ == "__main__":
