@@ -5,12 +5,14 @@
 from __future__ import annotations
 
 import logging
-from datetime import datetime
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, ClassVar
 
 import pytz
 
 from ring_doorbell.const import URL_DOORBELL_HISTORY, RingCapability
+from ring_doorbell.util import (
+    parse_datetime,
+)
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -43,11 +45,11 @@ class RingGeneric:
         """Return string representation of device."""
         return f"{self.name} ({self.kind})"
 
-    def update(self) -> None:
+    async def async_update(self) -> None:
         """Update this device info."""
-        self.update_health_data()
+        await self.async_update_health_data()
 
-    def update_health_data(self) -> None:
+    async def async_update_health_data(self) -> None:
         """Update the health data."""
         raise NotImplementedError
 
@@ -158,7 +160,7 @@ class RingGeneric:
         """Return the result of the last history query."""
         return self._last_history
 
-    def history(  # noqa: C901, PLR0912, PLR0913
+    async def async_history(  # noqa: C901, PLR0913, PLR0912
         self,
         *,
         limit: int = 30,
@@ -187,8 +189,7 @@ class RingGeneric:
 
         # set cap for max queries
         # pylint:disable=consider-using-min-builtin
-        if retry > 10:
-            retry = 10
+        retry = min(retry, 10)
 
         while True:
             params = {"limit": limit}
@@ -196,7 +197,8 @@ class RingGeneric:
                 params["older_than"] = older_than
 
             url = URL_DOORBELL_HISTORY.format(self.device_api_id)
-            response = self._ring.query(url, extra_params=params).json()
+            resp = await self._ring.async_query(url, extra_params=params)
+            response = resp.json()
 
             # cherrypick only the selected kind events
             if kind:
@@ -204,23 +206,11 @@ class RingGeneric:
 
             if convert_timezone:
                 # convert for specific timezone
-                utc = pytz.utc
                 if timezone:
                     mytz = pytz.timezone(timezone)
 
                 for entry in response:
-                    dt_at = datetime.strptime(
-                        entry["created_at"], "%Y-%m-%dT%H:%M:%S.%f%z"
-                    )
-                    utc_dt = datetime(
-                        dt_at.year,
-                        dt_at.month,
-                        dt_at.day,
-                        dt_at.hour,
-                        dt_at.minute,
-                        dt_at.second,
-                        tzinfo=utc,
-                    )
+                    utc_dt = parse_datetime(entry["created_at"])
                     if timezone:
                         tz_dt = utc_dt.astimezone(mytz)
                         entry["created_at"] = tz_dt
@@ -249,3 +239,29 @@ class RingGeneric:
 
         self._last_history = response
         return self._last_history
+
+    DEPRECATED_API_QUERIES: ClassVar = {
+        "history",
+        "update",
+        "update_health_data",
+    }
+    DEPRECATED_API_PROPERTY_GETTERS: ClassVar[set[str]] = set()
+    DEPRECATED_API_PROPERTY_SETTERS: ClassVar[set[str]] = set()
+
+    if not TYPE_CHECKING:
+
+        def __getattr__(self, name: str) -> Any:
+            """Get a deprecated attribute or raise an error."""
+            if name in self.DEPRECATED_API_QUERIES:
+                return self._ring.auth._dep_handler.get_api_query(self, name)  # noqa: SLF001
+            if name in self.DEPRECATED_API_PROPERTY_GETTERS:
+                return self._ring.auth._dep_handler.get_api_property(self, name)  # noqa: SLF001
+            msg = f"{self.__class__.__name__} has no attribute {name!r}"
+            raise AttributeError(msg)
+
+        def __setattr__(self, name: str, value: Any) -> None:
+            """Set a deprecated attribute or raise an error."""
+            if name in self.DEPRECATED_API_PROPERTY_SETTERS:
+                self._ring.auth._dep_handler.set_api_property(self, name, value)  # noqa: SLF001
+            else:
+                super().__setattr__(name, value)
