@@ -1,34 +1,25 @@
 """The tests for the Ring platform."""
 
-import asyncio
 import datetime
 import json
 
 import pytest
+from freezegun.api import FrozenDateTimeFactory
 from ring_doorbell import Ring
 from ring_doorbell.exceptions import RingError
-from ring_doorbell.listen import can_listen
+from ring_doorbell.listen import RingEventListener
 
-from tests.conftest import load_fixture
-
-# test_module.py
-pytestmark = pytest.mark.skipif(
-    can_listen is False, reason=("requires the extra [listen] to be installed")
-)
-
-if can_listen:
-    from ring_doorbell.listen import RingEventListener
+from tests.conftest import load_alert_v1, load_alert_v2, load_fixture
 
 
 async def test_listen(auth, mocker):
     import firebase_messaging
 
-    disconnectmock = mocker.patch("firebase_messaging.FcmPushClient.stop")
     ring = Ring(auth)
     listener = RingEventListener(ring)
 
-    listener.start()
-    assert firebase_messaging.FcmPushClient.checkin.call_count == 1
+    await listener.start()
+    assert firebase_messaging.FcmPushClient.checkin_or_register.call_count == 1
     assert firebase_messaging.FcmPushClient.start.call_count == 1
     assert listener.subscribed is True
     assert listener.started is True
@@ -45,7 +36,6 @@ async def test_listen(auth, mocker):
     cbid = listener.add_notification_callback(lambda: 2)
     del listener._callbacks[1]
     listener.remove_notification_callback(cbid)
-    disconnectmock.assert_called()
 
 
 async def test_active_dings(auth, mocker):
@@ -53,83 +43,82 @@ async def test_active_dings(auth, mocker):
 
     ring = Ring(auth)
     listener = RingEventListener(ring)
-    listener.start()
-    assert firebase_messaging.FcmPushClient.checkin.call_count == 1
+    await listener.start()
+    assert firebase_messaging.FcmPushClient.checkin_or_register.call_count == 1
     assert firebase_messaging.FcmPushClient.start.call_count == 1
     assert listener.subscribed is True
     assert listener.started is True
     num_active = len(ring.active_alerts())
-    assert num_active == 3
+    assert num_active == 0
     alertstoadd = 2
     for i in range(alertstoadd):
-        msg = json.loads(load_fixture("ring_listen_fcmdata.json"))
-        gcmdata_dict = json.loads(load_fixture("ring_listen_ding.json"))
-        created_at = datetime.datetime.utcnow().isoformat(timespec="milliseconds") + "Z"  # noqa: DTZ003
-        gcmdata_dict["ding"]["created_at"] = created_at
-        gcmdata_dict["ding"]["id"] = gcmdata_dict["ding"]["id"] + i
-        msg["data"]["gcmData"] = json.dumps(gcmdata_dict)
+        msg = load_alert_v1("doorbot_ding", 123456781, ding_id_inc=i)
+        listener._on_notification(msg, "1234567" + str(i))
+        msg = load_alert_v2("camera_motion", 123456782, ding_id_inc=i)
+        listener._on_notification(msg, "1234567" + str(i))
+        msg = load_alert_v1("intercom_unlock", 185036587, ding_id_inc=i)
         listener._on_notification(msg, "1234567" + str(i))
 
     dings = ring.active_alerts()
-    assert len(dings) == num_active + alertstoadd
+    assert len(dings) == num_active + alertstoadd * 3
     # Test with the same id which should overwrite
     # previous and keep the overall count the same
     for i in range(alertstoadd):
-        msg = json.loads(load_fixture("ring_listen_fcmdata.json"))
-        gcmdata_dict = json.loads(load_fixture("ring_listen_ding.json"))
-        created_at = datetime.datetime.utcnow().isoformat(timespec="milliseconds") + "Z"  # noqa: DTZ003
-        gcmdata_dict["ding"]["created_at"] = created_at
-        gcmdata_dict["ding"]["id"] = gcmdata_dict["ding"]["id"] + 1
-        msg["data"]["gcmData"] = json.dumps(gcmdata_dict)
+        msg = load_alert_v1("doorbot_ding", 123456781, ding_id_inc=i)
+        listener._on_notification(msg, "1234567" + str(i))
+        msg = load_alert_v2("camera_motion", 123456782, ding_id_inc=i)
         listener._on_notification(msg, "1234567" + str(i))
 
     dings = ring.active_alerts()
-    assert len(dings) == num_active + alertstoadd
-    listener.stop()
+    assert len(dings) == num_active + alertstoadd * 3
+    await listener.stop()
 
 
-async def test_intercom_unlock(auth, mocker):
-    import firebase_messaging
-
+async def test_ding_expirey(auth, mocker, freezer: FrozenDateTimeFactory):
     ring = Ring(auth)
     listener = RingEventListener(ring)
-    listener.start()
-    assert firebase_messaging.FcmPushClient.checkin.call_count == 1
-    assert firebase_messaging.FcmPushClient.start.call_count == 1
+    await listener.start()
+
     assert listener.subscribed is True
     assert listener.started is True
-    num_active = len(ring.active_alerts())
-    assert num_active == 3
+
+    assert len(ring.push_dings_data) == 0
+    assert len(ring.active_alerts()) == 0
+
     alertstoadd = 2
     for i in range(alertstoadd):
-        msg = json.loads(load_fixture("ring_listen_fcmdata.json"))
-        gcmdata_dict = json.loads(load_fixture("ring_listen_intercom_unlock.json"))
-        msg["data"]["gcmData"] = json.dumps(gcmdata_dict)
+        msg = load_alert_v1("doorbot_ding", 123456781, ding_id_inc=i)
+        listener._on_notification(msg, "1234567" + str(i))
+        msg = load_alert_v2("camera_motion", 123456782, ding_id_inc=i)
+        listener._on_notification(msg, "1234567" + str(i))
+        msg = load_alert_v1("intercom_unlock", 185036587, ding_id_inc=i)
         listener._on_notification(msg, "1234567" + str(i))
 
-    dings = ring.active_alerts()
-    assert len(dings) == num_active + alertstoadd
+    assert len(ring.push_dings_data) == 6
+    assert len(ring.active_alerts()) == 6
 
-    listener.stop()
+    freezer.tick(datetime.timedelta(minutes=5))
+
+    msg = load_alert_v1("doorbot_ding", 123456781, ding_id_inc=alertstoadd + 1)
+    listener._on_notification(msg, "123456781" + str(alertstoadd + 1))
+
+    assert len(ring.push_dings_data) == 1
+    assert len(ring.active_alerts()) == 1
 
 
-@pytest.mark.nolistenmock()
-async def test_listen_subscribe_fail(auth, mocker, requests_mock, caplog):
+@pytest.mark.nolistenmock
+async def test_listen_subscribe_fail(auth, mocker, caplog, putpatch_status_fixture):
     checkinmock = mocker.patch(
-        "firebase_messaging.FcmPushClient.checkin", return_value="foobar"
+        "firebase_messaging.FcmPushClient.checkin_or_register", return_value="foobar"
     )
     connectmock = mocker.patch("firebase_messaging.FcmPushClient.start")
     mocker.patch("firebase_messaging.FcmPushClient.is_started", return_value=True)
 
-    requests_mock.patch(
-        "https://api.ring.com/clients_api/device",
-        status_code=401,
-        content=b"foobar",
-    )
+    putpatch_status_fixture.overrides["https://api.ring.com/clients_api/device"] = 401
 
     ring = Ring(auth)
     listener = RingEventListener(ring)
-    listener.start()
+    await listener.start()
     # Check in gets and error so register is called
     assert checkinmock.call_count == 1
     assert listener.subscribed is False
@@ -138,7 +127,7 @@ async def test_listen_subscribe_fail(auth, mocker, requests_mock, caplog):
 
     exp = (
         "Unable to checkin to listen service, "
-        "response was 401 foobar, event listener not started"
+        "response was 401 , event listener not started"
     )
     assert (
         len(
@@ -152,22 +141,22 @@ async def test_listen_subscribe_fail(auth, mocker, requests_mock, caplog):
     )
 
 
-@pytest.mark.nolistenmock()
-async def test_listen_gcm_fail(auth, mocker, requests_mock, caplog):
+@pytest.mark.nolistenmock
+async def test_listen_gcm_fail(auth, mocker):
     # Check in gets and error so register is called, the subscribe gets an error
     credentials = json.loads(load_fixture("ring_listen_credentials.json"))
     checkinmock = mocker.patch(
-        "firebase_messaging.fcmpushclient.gcm_check_in", return_value=None
+        "firebase_messaging.fcmregister.FcmRegister.gcm_check_in", return_value=None
     )
     registermock = mocker.patch(
-        "firebase_messaging.FcmPushClient.register", return_value=credentials
+        "firebase_messaging.fcmregister.FcmRegister.register", return_value=credentials
     )
     connectmock = mocker.patch("firebase_messaging.FcmPushClient.start")
     mocker.patch("firebase_messaging.FcmPushClient.is_started", return_value=True)
 
     ring = Ring(auth)
     listener = RingEventListener(ring, credentials)
-    listener.start()
+    await listener.start()
     # Check in gets and error so register is called
     assert checkinmock.call_count == 1
     assert registermock.call_count == 1
@@ -176,24 +165,24 @@ async def test_listen_gcm_fail(auth, mocker, requests_mock, caplog):
     assert connectmock.call_count == 1
 
 
-@pytest.mark.nolistenmock()
-async def test_listen_fcm_fail(auth, mocker, requests_mock, caplog):
+@pytest.mark.nolistenmock
+async def test_listen_fcm_fail(auth, mocker, caplog):
     checkinmock = mocker.patch(
-        "firebase_messaging.FcmPushClient.checkin", return_value=None
+        "firebase_messaging.FcmPushClient.checkin_or_register", return_value=None
     )
     connectmock = mocker.patch("firebase_messaging.FcmPushClient.start")
     mocker.patch("firebase_messaging.FcmPushClient.is_started", return_value=True)
 
     ring = Ring(auth)
     listener = RingEventListener(ring)
-    listener.start()
+    await listener.start()
     # Check in gets and error so register is called
     assert checkinmock.call_count == 1
 
     assert listener.subscribed is False
     assert listener.started is False
     assert connectmock.call_count == 0
-    exp = "Unable to check in to fcm, event listener not started"
+    exp = "Ring listener unable to check in to fcm, event listener not started"
     assert (
         len(
             [
@@ -204,24 +193,3 @@ async def test_listen_fcm_fail(auth, mocker, requests_mock, caplog):
         )
         == 1
     )
-
-
-def test_no_event_loop(auth):
-    ring = Ring(auth)
-    listener = RingEventListener(ring)
-    listener.start()
-    ring.update_dings()
-    assert listener.started is True
-
-
-async def test_run_in_executor(auth):
-    import firebase_messaging
-
-    ring = Ring(auth)
-    listener = RingEventListener(ring)
-    loop = asyncio.get_running_loop()
-    await loop.run_in_executor(None, listener.start)
-
-    assert listener.started
-    assert firebase_messaging.FcmPushClient.checkin.call_count == 1
-    assert firebase_messaging.FcmPushClient.start.call_count == 1
